from .codec import Codec, load_file, classproperty
from .conformance_suite import (
    ConformanceSuite,
    get_suite,
    list_all_suites,
    list_loaded_suites,
    register_suite,
)
from .file import Attribute, DataType, File, Variable
from .loader import load_rules_from_dir
from .rule import RegisterRule, Rule
from .validation_result import Severity, ValidationResult, ValidationResultGroup
from .yaml_rules import register_yaml_rule

__all__ = ["ConformanceSuite", "Rule", "Severity", "ValidationResult", "register_suite", "load_rules_from_dir",
           "RegisterRule", "get_suite", "list_loaded_suites", "list_all_suites", "File", "Variable",
           "Attribute", "DataType",
           "Codec", "classproperty", "ValidationResultGroup", "load_file"]



from typing import Literal

from ...file import File
from ...validation_result import Severity, ValidationResult, ValidationResultGroup
from .base import BaseAssertion, resolve_path


class ExistsAssertion(BaseAssertion):
    check: Literal["exists"] = "exists"

    def evaluate(self, file: File) -> ValidationResult | ValidationResultGroup:
        matches = resolve_path(file, self.path)
        if matches:
            return ValidationResult(valid=True, reference="", severity=Severity.INFO,
                                    message=f"Path '{self.path}' exists in the file.", target=self.path)
        else:
            return ValidationResult(valid=False, reference="", severity=Severity.ERROR,
                                    message=f"Path '{self.path}' does not exist in the file.", target=self.path)


class NotExistsAssertion(BaseAssertion):
    check: Literal["not_exists"] = "not_exists"

    def evaluate(self, file: File) -> ValidationResult | ValidationResultGroup:
        matches = resolve_path(file, self.path)
        if matches:
            return ValidationResult(valid=False, reference="", severity=Severity.ERROR,
                                    message=f"Path '{self.path}' exists in the file but should not.", target=self.path)
        else:
            return ValidationResult(valid=True, reference="", severity=Severity.INFO,
                                    message=f"Path '{self.path}' does not exist in the file as expected.",
                                    target=self.path)



from typing import Literal, Any

from ...file import File, Variable
from ...validation_result import Severity, ValidationResult, ValidationResultGroup
from .base import BaseAssertion, resolve_path
from ...file import DataType


class IsTypeAssertion(BaseAssertion):
    check: Literal["is_type"] = "is_type"
    type: str

    def single_assertion(self, file: File, path: str, value: Any) -> ValidationResult:
        if not isinstance(value, DataType):
            if isinstance(value, Variable):
                value = value.data_type
            else:
                return ValidationResult(valid=False, reference="", severity=Severity.ERROR,
                                        message=f"Value at path '{path}' is not a valid DataType, got '{value}'.",
                                        target=path)
        expected_type = DataType(self.type)
        if value != expected_type:
            return ValidationResult(valid=False, reference="", severity=Severity.ERROR,
                                    message=f"Value at path '{path}' is of type '{value}', expected '{expected_type}'.",
                                    target=path)
        else:
            return ValidationResult(valid=True, reference="", severity=Severity.INFO,
                                    message=f"Value at path '{path}' is of expected type '{expected_type}'.",
                                    target=path)



from typing import Protocol

from .file import File


# https://stackoverflow.com/a/13624858
class classproperty(property):
    def __get__(self, owner_self, owner_cls):
        return self.fget(owner_cls)


class Codec(Protocol):
    _registry = {}

    def __init_subclass__(cls) -> None:
        super().__init_subclass__()
        for ext in cls.supported_extensions:
            if ext in cls._registry:
                raise ValueError(f"Extension '{ext}' is already registered by {cls._registry[ext].__name__}.")
            cls._registry[ext] = cls

    @classmethod
    def get_codec_for_extension(cls, extension: str) -> type["Codec"]:
        if extension in cls._registry:
            return cls._registry[extension]
        else:
            raise ValueError(f"No codec registered for extension '{extension}'.")

    @classproperty
    def supported_extensions(cls) -> list[str]:
        ...

    @staticmethod
    def load(path: str) -> File | None:
        ...


def load_file(path: str):
    extension = path.split(".")[-1]
    return Codec.get_codec_for_extension(extension).load(path)



from .contains_keys import ContainsKeysAssertion
from .matches import MatchesAssertion
from .is_type import IsTypeAssertion
from .combinations import AllOf, AnyOf, NoneOf, Not, IfThen, IfThenElse, OneOf, AtLeast, AtMost, Exactly
from .exists import ExistsAssertion, NotExistsAssertion
from .comparisons import ComparisonAssertion, RangeAssertion
from .collections import ContainsAssertion, NotContainsAssertion, LengthAssertion, NotEmptyAssertion

__all__ = [
    "ContainsKeysAssertion",
    "MatchesAssertion",
    "IsTypeAssertion",
    "AllOf",
    "AnyOf",
    "NoneOf",
    "Not",
    "IfThen",
    "IfThenElse",
    "OneOf",
    "AtLeast",
    "AtMost",
    "Exactly",
    "ExistsAssertion",
    "NotExistsAssertion",
    "ComparisonAssertion",
    "RangeAssertion",
    "ContainsAssertion",
    "NotContainsAssertion",
    "LengthAssertion",
    "NotEmptyAssertion",
]



from typing import Literal, Any

from ...file import File
from ...validation_result import Severity, ValidationResult, ValidationResultGroup
from .base import BaseAssertion, resolve_path


# Value references another variable name
class ReferencesVariableAssertion(BaseAssertion):
    check: Literal["reference_variable"] = "reference_variable"
    variable: str

    def single_assertion(self, file: File, path: str, value: Any) -> ValidationResult:
        if type(value) is str:
            if value in file.variables:
                return ValidationResult(valid=True, reference=file.variables[value], severity=Severity.INFO,
                                        message=f"Value at path '{path}' references variable '{value}'.",
                                        target=self.path)
            else:
                return ValidationResult(valid=False, reference="", severity=Severity.ERROR,
                                        message=f"Value at path '{path}' references variable '{value}', which is not defined.",
                                        target=self.path)
        else:
            return ValidationResult(valid=False, reference="", severity=Severity.ERROR,
                                    message=f"Value at path '{path}' is not a string and cannot reference a variable.",
                                    target=self.path)



"""Configuration management for AstraLint."""

from .schema import AstraLintConfig, OutputConfig
from .loader import load_config, find_config_file, find_project_root, validate_config_file

__all__ = [
    "AstraLintConfig",
    "OutputConfig",
    "load_config",
    "find_config_file",
    "find_project_root",
    "validate_config_file",
]

